@@ -1,4 +1,4 @@
-/* ulogd_MYSQL.c, Version $Revision: 1.13 $
+/* ulogd_MYSQL.c, Version $Revision: 705 $
  *
  * ulogd output plugin for logging to a MySQL database
  *
@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  * 
- * $Id: ulogd_MYSQL.c,v 1.13 2003/08/23 17:37:53 laforge Exp $
+ * $Id: ulogd_MYSQL.c 705 2005-02-16 11:25:09Z laforge $
  *
  * 15 May 2001, Alex Janssen <alex@ynfonatic.de>:
  *      Added a compability option for older MySQL-servers, which
@@ -29,10 +29,13 @@
  *	See ulogd/doc/mysql.table.ipaddr-as-string as an example.
  *	BE WARNED: This has _WAY_ less performance during table searches.
  *
+ * 09 Feb 2005, Sven Schuster <schuster.sven@gmx.de>:
+ * 	Added the "port" parameter to specify ports different from 3306
  */
 
 #include <stdlib.h>
 #include <string.h>
+#include <arpa/inet.h>
 #include <ulogd/ulogd.h>
 #include <ulogd/conffile.h>
 #include <mysql/mysql.h>
@@ -65,31 +68,55 @@ static char *stmt_val;
 static char *stmt_ins;
 
 /* our configuration directives */
-static config_entry_t db_ce = { NULL, "mysqldb", CONFIG_TYPE_STRING,
-				CONFIG_OPT_MANDATORY, 0,
-				{ } };
+static config_entry_t db_ce = { 
+	.key = "db", 
+	.type = CONFIG_TYPE_STRING,
+	.options = CONFIG_OPT_MANDATORY,
+};
 
-static config_entry_t host_ce = { &db_ce, "mysqlhost", CONFIG_TYPE_STRING,
-				CONFIG_OPT_MANDATORY, 0,
-				{ } };
+static config_entry_t host_ce = { 
+	.next = &db_ce, 
+	.key = "host", 
+	.type = CONFIG_TYPE_STRING,
+	.options = CONFIG_OPT_MANDATORY,
+};
 
-static config_entry_t user_ce = { &host_ce, "mysqluser", CONFIG_TYPE_STRING,
-				CONFIG_OPT_MANDATORY, 0,
-				{ } };
+static config_entry_t user_ce = { 
+	.next = &host_ce, 
+	.key = "user", 
+	.type = CONFIG_TYPE_STRING,
+	.options = CONFIG_OPT_MANDATORY,
+};
 
-static config_entry_t pass_ce = { &user_ce, "mysqlpass", CONFIG_TYPE_STRING,
-				CONFIG_OPT_MANDATORY, 0,
-				{ } };
+static config_entry_t pass_ce = { 
+	.next = &user_ce, 
+	.key = "pass", 
+	.type = CONFIG_TYPE_STRING,
+	.options = CONFIG_OPT_MANDATORY,
+};
 
-static config_entry_t table_ce = { &pass_ce, "mysqltable", CONFIG_TYPE_STRING,
-				CONFIG_OPT_MANDATORY, 0,
-				{ } };
+static config_entry_t table_ce = { 
+	.next = &pass_ce, 
+	.key = "table", 
+	.type = CONFIG_TYPE_STRING,
+	.options = CONFIG_OPT_MANDATORY,
+};
+
+static config_entry_t port_ce = {
+	.next = &table_ce,
+	.key = "port",
+	.type = CONFIG_TYPE_INT,
+};
 
 /* our main output function, called by ulogd */
-static int _mysql_output(ulog_iret_t *result)
+static int mysql_output(ulog_iret_t *result)
 {
 	struct _field *f;
 	ulog_iret_t *res;
+#ifdef IP_AS_STRING
+	char *tmpstr;		/* need this for --log-ip-as-string */
+	struct in_addr addr;
+#endif
 
 	stmt_ins = stmt_val;
 
@@ -129,8 +156,10 @@ static int _mysql_output(ulog_iret_t *result)
 				break;
 			case ULOGD_RET_IPADDR:
 #ifdef IP_AS_STRING
+				memset(&addr, 0, sizeof(addr));
+				addr.s_addr = ntohl(res->value.ui32);
 				*stmt_ins++ = '\'';
-				tmpstr = inet_ntoa(ntohl(res->value.ui32));
+				tmpstr = inet_ntoa(addr);
 #ifdef OLD_MYSQL
 				mysql_escape_string(stmt_ins, tmpstr,
 						    strlen(tmpstr));
@@ -167,6 +196,11 @@ static int _mysql_output(ulog_iret_t *result)
 				sprintf(stmt_ins, "',");
 			/* sprintf(stmt_ins, "'%s',", res->value.ptr); */
 				break;
+			case ULOGD_RET_RAW:
+				ulogd_log(ULOGD_NOTICE,
+					"%s: type RAW not supported by MySQL\n",
+					res->key);
+				break;
 			default:
 				ulogd_log(ULOGD_NOTICE,
 					"unknown type %d for %s\n",
@@ -193,7 +227,7 @@ static int _mysql_output(ulog_iret_t *result)
 #define MYSQL_VALSIZE	100
 
 /* create the static part of our insert statement */
-static int _mysql_createstmt(void)
+static int mysql_createstmt(void)
 {
 	struct _field *f;
 	unsigned int size;
@@ -245,7 +279,7 @@ static int _mysql_createstmt(void)
 }
 
 /* find out which columns the table has */
-static int _mysql_get_columns(const char *table)
+static int mysql_get_columns(const char *table)
 {
 	MYSQL_RES *result;
 	MYSQL_FIELD *field;
@@ -294,40 +328,53 @@ static int _mysql_get_columns(const char *table)
 }
 
 /* make connection and select database */
-static int _mysql_open_db(char *server, char *user, char *pass, char *db)
+static int mysql_open_db(char *server, int port, char *user, char *pass, 
+			 char *db)
 {
 	dbh = mysql_init(NULL);
 	if (!dbh)
 		return 1;
 
-	if (!mysql_real_connect(dbh, server, user, pass, db, 0, NULL, 0))
+	if (!mysql_real_connect(dbh, server, user, pass, db, port, NULL, 0))
 		return 1;
 
 	return 0;
 }
 
-static ulog_output_t _mysql_plugin = { NULL, "mysql", &_mysql_output, NULL };
-
-void _init(void) 
+static int _mysql_init(void)
 {
-	/* register our configfile options here */
-	config_register_key(&table_ce);
-
 	/* have the opts parsed */
-	config_parse_file(0);
+	config_parse_file("MYSQL", &port_ce);
 
-	if (_mysql_open_db(host_ce.u.string, user_ce.u.string, 
+	if (mysql_open_db(host_ce.u.string, port_ce.u.value, user_ce.u.string, 
 			   pass_ce.u.string, db_ce.u.string)) {
 		ulogd_log(ULOGD_ERROR, "can't establish database connection\n");
-		return;
+		return -1;
 	}
 
 	/* read the fieldnames to know which values to insert */
-	if (_mysql_get_columns(table_ce.u.string)) {
+	if (mysql_get_columns(table_ce.u.string)) {
 		ulogd_log(ULOGD_ERROR, "unable to get mysql columns\n");
-		return;
+		return -1;
 	}
-	_mysql_createstmt();
-	register_output(&_mysql_plugin);
+	mysql_createstmt();
 
+	return 0;
+}
+
+static void _mysql_fini(void)
+{
+	mysql_close(dbh);
+}
+
+static ulog_output_t mysql_plugin = { 
+	.name = "mysql", 
+	.output = &mysql_output, 
+	.init = &_mysql_init,
+	.fini = &_mysql_fini,
+};
+
+void _init(void) 
+{
+	register_output(&mysql_plugin);
 }
